package org.nakedobjects.nos.remote.xstream;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.StreamCorruptedException;
import java.net.SocketException;

import org.apache.log4j.Logger;
import org.nakedobjects.nos.remote.command.ConnectionException;
import org.nakedobjects.nos.remote.command.Request;
import org.nakedobjects.nos.remote.command.socket.ClientConnection;

import com.thoughtworks.xstream.XStream;


class XStreamClientConnection extends ClientConnection {
    private static final Logger LOG = Logger.getLogger(XStreamClientConnection.class);
    private ObjectInputStream input;
    private ObjectOutputStream output;


    protected void openStreams(final InputStream input, final OutputStream output) throws IOException {
        this.input = new ObjectInputStream(input);
        this.output = new ObjectOutputStream(output);
    }

    protected Object request(final Request request) throws IOException {
        XStream xstream = new XStream();
        String requestData = xstream.toXML(request);
        LOG.info("sending " + requestData.length() + " bytes of data");
        try {
            LOG.debug("sending request \n" + requestData);
            try {
                output.writeObject(requestData);
            } catch (SocketException e) {
                reconnect();
                output.writeObject(requestData);
            }
            output.flush();
            String responseData = (String) input.readObject();
            LOG.debug("response received: \n" + responseData);
            return xstream.fromXML(responseData);

        } catch (StreamCorruptedException e) {
            try {
                int available = input.available();
                LOG.debug("error in reading; skipping bytes: " + available);
                input.skip(available);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            throw new ConnectionException(e.getMessage(), e);
        } catch (ClassNotFoundException e) {
            throw new ConnectionException("Failed request", e);
        }
    }

    
}
// Copyright (c) Naked Objects Group Ltd.
